-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'Find Closest Road Of Path (Mod)'
C.description = 'Returns the closest road data along the path from a given position.'
C.color = im.ImVec4(0.6, 0.5, 0, 0.8)
C.category = 'repeat_instant'

C.pinSchema = {
  { dir = 'in', type = 'flow', name = 'flow' },
  { dir = 'in', type = 'table', name = 'navPathData', description = 'Map path data.' },
  { dir = 'in', type = 'vec3', name = 'position', description = 'Position to query with the map path.' },

  { dir = 'out', type = 'flow', name = 'flow' },
  { dir = 'out', type = 'string', name = 'node1', description = 'First map node.' },
  { dir = 'out', type = 'string', name = 'node2', description = 'Second map node.' },
  { dir = 'out', type = 'vec3', name = 'position', description = 'Interpolated position.' },
  { dir = 'out', type = 'number', name = 'distance', description = 'Distance between original position and final position.' }
}

function C:init()
  self:reset()
end

function C:reset()
  self.aiPath = nil
  self.pinOut.flow.value = false
end

function C:_executionStopped()
  self:reset()
end

function C:work()
  if self.pinIn.flow.value then
    if self.pinIn.navPathData.value then
      local mapNodes = map.getMap().nodes
      self.aiPath = self.pinIn.navPathData.value
      local bestIdx, bestDist = 0, math.huge
      local pos = vec3(self.pinIn.position.value)
      local xnorm = 0
      local count = #self.aiPath

      for i, v in ipairs(self.aiPath) do
        local distSq = pos:squaredDistance(mapNodes[v].pos)
        if distSq < bestDist then
          bestDist = distSq
          bestIdx = i
        end
      end

      local n1, n2, n3 = self.aiPath[bestIdx], self.aiPath[math.min(count, bestIdx + 1)], self.aiPath[math.max(1, bestIdx - 1)]
      local p1, p2, p3 = mapNodes[n1].pos, mapNodes[n2].pos, mapNodes[n3].pos
      local xnorm1, xnorm2 = pos:xnormOnLine(p1, p2), pos:xnormOnLine(p3, p1)
      if bestIdx == count or xnorm1 < 0 or xnorm1 > 1 then
        n1, n2 = n3, n1
        p1, p2 = p3, p1
        xnorm = xnorm2
      else
        xnorm = xnorm1
      end

      local targetPos = linePointFromXnorm(p1, p2, clamp(xnorm, 0, 1))
      self.pinOut.node1.value = n1
      self.pinOut.node2.value = n2
      self.pinOut.position.value = targetPos:toTable()
      self.pinOut.distance.value = pos:distance(targetPos)
    end
    self.pinOut.flow.value = true
  end
end

return _flowgraph_createNode(C)