-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'Get Map Path Point (Mod)'
C.description = 'Returns data of the point ahead along a given path.'
C.color = im.ImVec4(0.6, 0.5, 0, 0.8)
C.category = 'once_instant'

C.pinSchema = {
  { dir = 'in', type = 'flow', name = 'flow' },
  { dir = 'in', type = 'flow', name = 'reset', impulse = true },
  { dir = 'in', type = 'table', name = 'navPathData', description = 'Map path data.' },
  { dir = 'in', type = 'string', name = 'startNode', hidden = true, description = 'Path node to start searching from.' },
  { dir = 'in', type = 'number', name = 'distance', description = 'Distance ahead along the map path to get the results from.' },
  { dir = 'in', type = 'bool', name = 'reverseDir', hidden = true, description = 'If true, search for a point in the opposite direction.' },

  { dir = 'out', type = 'flow', name = 'flow'},
  { dir = 'out', type = 'string', name = 'node1', description = 'First map node.' },
  { dir = 'out', type = 'string', name = 'node2', description = 'Second map node.' },
  { dir = 'out', type = 'vec3', name = 'position', description = 'Interpolated position.' },
  { dir = 'out', type = 'quat', name = 'rotation', description = 'Node-to-node rotation.' },
  { dir = 'out', type = 'number', name = 'radius', description = 'Node radius.' }
}

function C:init()
  self:reset()
end

function C:reset()
  self.aiPath = nil
  self.pinOut.flow.value = false
end

function C:_executionStopped()
  self:reset()
end

function C:work()
  if self.pinIn.reset.value then
    self:reset()
  end
  if self.pinIn.flow.value and not self.pinOut.flow.value then
    if self.pinIn.navPathData.value then
      local mapNodes = map.getMap().nodes
      self.aiPath = deepcopy(self.pinIn.navPathData.value)

      if self.pinIn.reverseDir.value == true then
        local temp = {}
        for i = #self.aiPath, 1, -1 do
          table.insert(temp, self.aiPath[i])
        end
        self.aiPath = temp
      end

      if self.pinIn.startNode.value then
        local index = arrayFindValueIndex(self.aiPath, self.pinIn.startNode.value)
        if index and index > 1 then
          for i = 1, index - 1 do
            table.remove(self.aiPath, 1)
          end
        end
      end

      local n1, n2, xnorm = map.getNodesFromPathDist(self.aiPath, self.pinIn.distance.value or 0)
      local p1, p2 = mapNodes[n1].pos, mapNodes[n2].pos
      self.pinOut.node1.value = n1
      self.pinOut.node2.value = n2
      self.pinOut.position.value = linePointFromXnorm(p1, p2, xnorm):toTable()
      self.pinOut.rotation.value = quatFromDir((p2 - p1):normalized(), vec3(0, 0, 1)):toTable()
      self.pinOut.radius.value = lerp(mapNodes[n1].radius, mapNodes[n2].radius, xnorm)

      self.pinOut.flow.value = true
    end
  end
end

return _flowgraph_createNode(C)